﻿package MT::Plugin::ExtendedEntries;
use strict;
use MT;
use MT::Plugin;
use MT::Entry;
use ExtFields::Extfields;

our $VERSION = '0.2';

use base qw( MT::Plugin );

@MT::Plugin::ExtendedEntries::ISA = qw( MT::Plugin );

my $plugin = new MT::Plugin::ExtendedEntries( {
    id => 'ExtendedEntries',
    key => 'ExtendedEntries',
    description => 'MTEntries with extra fields.',
    name => 'ExtendedEntries',
    author_name => 'Alfasado,Inc.',
    author_link => 'http://alfasado.net/',
    'version' => $VERSION,
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        tags => {
            block => {
                'ExtendedEntries' => \&_hndr_extended_entries,
                'ExtendedEntriesHeader' => \&_pass_tokens,
                'ExtendedEntriesFooter' => \&_pass_tokens,
            },
        }
   } );
}

sub _hndr_extended_entries {
    my ( $ctx, $args, $cond ) = @_;
    my $tokens = $ctx->stash( 'tokens' );
    my $builder = $ctx->stash( 'builder' );
    my $blog = $ctx->stash( 'blog' );
    my $label = $args->{ label };
    my $blog_ids = $args->{ blog_ids };
    my $lastn = $args->{ lastn };
    $lastn = 100 unless $lastn;
    my $sort_by = $args->{ sort_by };
    $sort_by = 'authored_on' unless $sort_by;
    my $sort_order = $args->{ sort_order };
    my $direction = 'ascend';
    $direction = $sort_order if $sort_order;
    my $eq = $args->{ eq };
    my $blog_id;
    if ( $blog_ids ) {
        my @bids = split( /,/, $blog_ids );
        $blog_id = [ @bids ];
    } else {
        $blog_id = $blog->id;
    }
    my $terms = { status => MT::Entry::RELEASE() };
    $terms->{ blog_id } = $blog_id if $blog_id;
    my @entries = MT::Entry->load( $terms,
                           { sort => $sort_by,
                             limit => $lastn,
                             direction => $direction,
                             'join' => [ 'ExtFields::Extfields', 'entry_id',
                                       { label => $label, text => $eq, },
                                       { unique => 1 } ],
                           } );
    my $res = '';
    my $i = 0;
    for my $entry ( @entries ) {
        local $ctx->{__stash}{'entry'} = $entry;
        local $ctx->{__stash}{'blog'} = $entry->blog;
        local $ctx->{__stash}{'blog_id'} = $entry->blog_id;
        my $out = $builder->build( $ctx, $tokens, {
            %$cond,
            ExtendedEntriesHeader => $i == 0,
            ExtendedEntriesFooter => !defined( $entries[$i + 1] ),
        });
        if ( !defined( $out ) ) { return $ctx->error( $builder->errstr ) };
        $res .= $out;
        $i++;
    }
    $res;
}

sub _pass_tokens {
    my ( $ctx, $args, $cond ) = @_;
    $ctx->stash( 'builder' )->build($ctx, $ctx->stash( 'tokens' ), $cond );
}

1;